'use strict';

var path = require('path');
var gutil = require('gulp-util');
var through = require('through2');
var vcl = require('vcl-preprocessor');
var PluginError = gutil.PluginError;

// consts
var PLUGIN_NAME = "gulp-vcl-preprocessor";

// plugin level function (dealing with files)
function gulpVclPreprocessor(opts) {
  opts = opts || {};

  // creating a stream through which each file will pass
  var stream = through.obj(function(file, enc, cb) {

    if (file.isBuffer()) {
      var compiler = null;
      var fallBackName = 'style.css';
      if (opts.package === true) {
        // build a module
        try {
          var packagejson = JSON.parse(file.contents.toString('utf8'));
          compiler = vcl.package(packagejson, opts);
        } catch (e) {
          this.emit('error', e);
        }
      } else {
        var styl = file.contents.toString('utf8'); // vcl/reworkcss uses strings
        opts.source = file.path;
        compiler = vcl(styl, opts);
        fallBackName = path.basename(file.path, 'styl') + '.css';
      }
      file.contents = new Buffer(compiler.toString());

      file.path = opts.output || fallBackName;
    }

    if (file.isStream()) {
      var err = new PluginError(PLUGIN_NAME, 'Streams are not supported!');
      this.emit('error', err);
    }

    this.push(file);

    return cb();
  });

  // returning the file stream
  return stream;
}

// exporting the plugin main function
module.exports = gulpVclPreprocessor;
